@@ -1,12 +1,16 @@
 """deserialize icd hierarchies computed in datacleaning.py"""
 
-from typing import Literal, Optional, Sequence, Tuple
+from typing import Optional, Sequence, Tuple
 import json
-import importlib
 from datetime import datetime
 import networkx as nx
 from . import data
 
+try:
+    import importlib.resources as importlib_resources
+except ModuleNotFoundError:
+    import importlib_resources
+
 
 def icd9() -> Tuple[nx.Graph, Sequence[str]]:
     """deserialize icd9 hierarchy
@@ -14,18 +18,15 @@ def icd9() -> Tuple[nx.Graph, Sequence[str]]:
     Returns:
         Tuple[nx.Graph, Sequence[str]]: ICD9 hierarchy and codes
     """
-    with importlib.resources.open_binary(data, "icd-9-hierarchy.json") as f:
-        hierarchy = f.read().decode("utf-8")
-        hierarchy = json.loads(hierarchy)
+    with importlib_resources.open_text(data, "icd-9-hierarchy.json") as f:
+        hierarchy = json.load(f)
     return (
         nx.readwrite.json_graph.node_link_graph(hierarchy["graph"]),
         hierarchy["codes"],
     )
 
 
-def icd10cm(
-    version: Optional[Literal["2019", "2020", "2020"]] = None
-) -> Tuple[nx.Graph, Sequence[str]]:
+def icd10cm(version: Optional[str] = None) -> Tuple[nx.Graph, Sequence[str]]:
     """deserialize icd-10-cm hierarchy
 
     Args:
@@ -36,10 +37,10 @@ def icd10cm(
         Tuple[nx.Graph, Sequence[str]]: ICD-10-CM hierarchy and codes
     """
     if version is None:
-        version = datetime.now().year
-    with importlib.resources.open_binary(data, f"icd-10-{version}-hierarchy.json") as f:
-        hierarchy = f.read().decode("utf-8")
-        hierarchy = json.loads(hierarchy)
+        version = str(datetime.now().year)
+    assert version in ["2019", "2020", "2021"]
+    with importlib_resources.open_text(data, f"icd-10-{version}-hierarchy.json") as f:
+        hierarchy = json.loads(f.read())
     return (
         nx.readwrite.json_graph.node_link_graph(hierarchy["graph"]),
         hierarchy["codes"],
